﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=7.8
@EndOfDesignText@
Sub Process_Globals

End Sub


public Sub IndexOfDoubleQuote(text As String, start As Int) As Int
	Return IndexOfQuote(text, start, Chr(34))
End Sub


public Sub IndexOfLineEnd(text As String, start As Int) As Int

	' Find the next end-of-line character, starting at index 'start'.
	' Return -1 if none found.
	If (text.Length = 0) Then
		Return -1
	End If

	If (start >= text.Length) Then
		Return -1
	End If

	For i = start To (text.Length - 1)

		Select text.CharAt(i)
			Case Chr(10)
				Return i
		End Select

	Next

	Return -1

End Sub


public Sub IndexOfNonWhitespace(text As String, start As Int) As Int

	' Find the next non-whitespace character, starting at index 'start'.
	' Return -1 if none found.
	If (text.Length = 0) Then
		Return -1
	End If

	If (start >= text.Length) Then
		Return -1
	End If

	For i = start To (text.Length - 1)

		Select text.CharAt(i)

			Case " ", Chr(9)

			Case Else
				Return i

		End Select

	Next

	Return -1

End Sub


public Sub IndexOfQuote(text As String, start As Int, a_quote As Char) As Int

	' Find the next quote character, starting at index 'start'.
	' Return -1 if none found.
	If (text.Length = 0) Then
		Return -1
	End If

	If (start >= text.Length) Then
		Return -1
	End If

	Dim result As Int = -1
	Dim i As Int = start

	Do While True

		If (text.CharAt(i) = a_quote) Then
			' Are we at the last character?
			If (i = text.Length - 1) Then
				result = i
				Exit
			End If

			' Is the next one a quote also?
			If (text.CharAt(i + 1) = a_quote) Then
				' Yes - move on.
				i = i + 1
			End If

			result = i
			Exit
		End If

		If (i >= text.Length - 1) Then Exit
		i = i + 1

	Loop

	Return result

End Sub


public Sub IndexOfSingleQuote(text As String, start As Int) As Int
	Return IndexOfQuote(text, start, Chr(39))
End Sub


public Sub IndexOfWhitespace(text As String, start As Int) As Int

	'  Find the next whitespace character, starting at index 'start'.
	' Return -1 if none found.
	If (text.Length = 0) Then
		Return -1
	End If

	If (start >= text.Length) Then
		Return -1
	End If

	For i = start To (text.Length - 1)

		Select text.CharAt(i)

			Case " ", Chr(9)
				Return i

		End Select

	Next

	Return -1

End Sub


public Sub IsNullOrWhitespace(text As String) As Boolean

	If (text.CompareTo(Null) = 0) Then Return True

	Select text.trim()
		Case "", Chr(9)
			Return True
	End Select

	Return False

End Sub


public Sub SplitText(text As String, maxFields As Int, allowCommas As Boolean, trimFields As Boolean) As List

	' Split 'text' on whitespace, allowing for quote-delimited fields (matching single or double quotes).
	' If 'maxFields' fields are present, the last field returned (i.e. filed 'maxFields - 1')
	' has the remainder of 'text' in it.
	Dim result As List
	result.Initialize

	If IsNullOrWhitespace(text) Then Return result

	Dim dquote As Char = Chr(34)
	Dim endPos As Int = 0
	Dim field As String
	Dim pos As Int = 0
	Dim squote As Char = Chr(39)

	Do While (pos <= text.Length - 1)

		If (result.Size = maxFields - 1) Then
			' Get the remainder of 'text' and finish.
			field = text.SubString(pos)
			If trimFields Then field = field.Trim
			result.Add(field)
			Exit				
		End If

		' Find the start of the next field.
		pos = IndexOfNonWhitespace(text, pos)
		If (pos < 0) Then Exit

		' The next field starts at 'pos'.

		' Does it start with a double quote?
		If (text.CharAt(pos) = dquote) Then
			' Yes - find the closing double quote, ignoring any embedded pairs of double quotes.
			endPos = IndexOfDoubleQuote(text, pos + 1)		

			If (endPos < 0) Then
				' Invalid.
				Log("ERROR - Invalid text quoting: '" & text & "'")
				Exit
			End If

			field = text.SubString2(pos + 1, endPos)						' strip the opening and closing double quotes
			If trimFields Then field = field.Trim
			result.Add(field)

			pos = endPos + 1
			Continue
		End If

		' Does it start with a single quote?
		If (text.CharAt(pos) = squote) Then
			' Yes - find the closing single quote, ignoring any embedded pairs of single quotes.
			endPos = IndexOfSingleQuote(text, pos + 1)

			If (endPos < 0) Then
				' Invalid.
				Log("ERROR - Invalid text quoting: '" & text & "'")
				Exit
			End If

			field = text.SubString2(pos + 1, endPos)						' strip the opening and closing single quotes
			If trimFields Then field = field.Trim
			result.Add(field)

			pos = endPos + 1
			Continue
		End If

		' Find the end of this field, i.e. the next whitespace character or the end of the text.
		endPos = IndexOfWhitespace(text, pos + 1)

		If (endPos < 0) Then
			' End of the text.
			field = text.SubString(pos)
			If trimFields Then field = field.Trim
			result.Add(field)
			Exit
		End If

		field = text.SubString2(pos, endPos)
		If trimFields Then field = field.Trim
		result.Add(field)
		pos = endPos + 1

	Loop

	Return result

End Sub


public Sub SplitTextLines(text As String, trimLines As Boolean) As List

	Dim result As List
	result.Initialize

	If IsNullOrWhitespace(text) Then Return result

	Dim endPos As Int = 0
	Dim line As String
	Dim pos As Int = 0

	Do While (pos <= text.Length - 1)

		' Find the end of this line, i.e. the next end-of-line or the end of the text.
		endPos = IndexOfLineEnd(text, pos)

		If (endPos < 0) Then
			' End of the text.
			line = text.SubString(pos)
			If trimLines Then line = line.trim
			result.Add(line)
			Exit
		End If

		line = text.SubString2(pos, endPos)
		If trimLines Then line = line.trim
		result.Add(line)

		pos = endPos + 1

	Loop

	Return result

End Sub

